/************************************************************************
 *                                                                      *
 * Project: KAF - Khelben Arcade Frontend                               *
 *                                                                      *
 * Author: Marco Mastroddi <marco.mastroddi@libero.it>                  *
 *                                                                      *
 ************************************************************************
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 ************************************************************************/

/*
 *    File    : wndmainwindow.cpp
 */




#include "wndmainwindow.h"
#include "wndbuildgameslist.h"
#include "wndmamepaths.h"
#include "wndgameoptions.h"
#include "wndpreferences.h"
#include "wndmameprocess.h"
#include "wndinfo.h"
#include "gameslist.h"
#include "game.h"
#include "gameitem.h"
#include "options.h"
#include "pixmaps.h"
#include "mameprocess.h"

#include <qstatusbar.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qtimer.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qgroupbox.h>
#include <qprogressdialog.h>
#include <qpushbutton.h>
#include <qcombobox.h>


const QString KafDefaultTitle = "KAF - Khelben's Arcade Frontend";



WndMainWindow::WndMainWindow( QWidget* parent )
  : GuiMainWindow( parent, "KAF Main Window" )
{
  initGamesListView();

  setIcon( Pixmaps::getMameIcon() );
  labelPreview->setPixmap( Pixmaps::getMameBigIcon() );

  setCaption( KafDefaultTitle );

  mp_info = 0;
  mp_progress = 0;
  m_progressCancelled = false;
  mp_options = new Options();
  mp_games = new GamesList( lvGames );
  mp_games->loadFavorites( mp_options->mameFavoritesFilePath() );

  QPopupMenu *pGeneralMenu = new QPopupMenu( this, "menu general" );

  mp_actStartGame = new QAction( trUtf8( "Start game" ), trUtf8( "Start game..." ),
                                 0, this, "actStartGame" );
  connect( mp_actStartGame, SIGNAL( activated() ), this, SLOT( slotStartGame() ) );
  mp_actStartGame->addTo( pGeneralMenu );

  pGeneralMenu->insertSeparator();
  int id = pGeneralMenu->insertItem( trUtf8( "Preferences..." ), this, SLOT( slotShowPreferences() ) );
  pGeneralMenu->setAccel( Qt::Key_F2, id );
  pGeneralMenu->insertSeparator();
  pGeneralMenu->insertItem( trUtf8( "Close" ), this, SLOT( close() ) );

  QPopupMenu *pHelpMenu = new QPopupMenu( this, "menu help" );
  pHelpMenu->insertItem( trUtf8( "Information..." ), this, SLOT( slotShowMameInfo() ) );
  pHelpMenu->insertItem( trUtf8( "About..." ), this, SLOT( slotShowAbout() ) );

  mp_filterMenu = new QPopupMenu( this, "menu filter" );
  mp_filterMenu->setCheckable( true );
  mp_filterMenu->insertItem( trUtf8( "All" ), 0 );
  mp_filterMenu->insertItem( trUtf8( "Availables" ), Game::Available );
  mp_filterMenu->insertItem( trUtf8( "Unavailables" ), Game::Unavailable );
  mp_filterMenu->insertItem( trUtf8( "Favorites" ), Game::Favorite );
  mp_filterMenu->insertItem( trUtf8( "NeoGeo" ), Game::NeoGeo );
  mp_filterMenu->insertItem( trUtf8( "Originals" ), Game::Original );
  mp_filterMenu->insertItem( trUtf8( "Clones" ), Game::Clone );
  mp_filterMenu->insertItem( trUtf8( "Raster" ), Game::Raster );
  mp_filterMenu->insertItem( trUtf8( "Vector" ), Game::Vector );

  mp_filterMenu->setItemChecked( mp_games->filter(), true );

  connect( mp_filterMenu, SIGNAL( activated( int ) ), this, SLOT( slotChangeFilter( int ) ) );

  mp_mameMenu = new QPopupMenu( this, "menu mame" );

  mp_actCheckMameVersion = new QAction( trUtf8( "Check version" ),
                                 trUtf8( "Check version" ),
                                 0, this, "actCheckMameVersion" );
  mp_actCheckMameVersion->setAccel( Qt::Key_F1 );
  connect( mp_actCheckMameVersion, SIGNAL( activated() ), this, SLOT( slotCheckMameVersion() ) );
  mp_actCheckMameVersion->addTo( mp_mameMenu );

  mp_actBuildList = new QAction( trUtf8( "Build games list" ),
                                 trUtf8( "Build games list..." ),
                                 0, this, "actBuildGamesList" );
  mp_actBuildList->setAccel( SHIFT + Qt::Key_F3 );
  connect( mp_actBuildList, SIGNAL( activated() ), this, SLOT( slotBuildGamesListActivated() ) );
  mp_actBuildList->addTo( mp_mameMenu );

  mp_actSearchAvailables = new QAction( trUtf8( "Search for available games" ),
                                 trUtf8( "Search for available games" ),
                                 0, this, "actSearchAvailables" );
  mp_actSearchAvailables->setAccel( Qt::Key_F5 );
  connect( mp_actSearchAvailables, SIGNAL( activated() ), this, SLOT( slotSearchForAvailablesActivated() ) );
  mp_actSearchAvailables->addTo( mp_mameMenu );

  mp_actReadMameInfo = new QAction( trUtf8( "Read games information" ),
                                 trUtf8( "Read games information" ),
                                 0, this, "actReadMameInfo" );
  mp_actReadMameInfo->setAccel( Qt::Key_F6 );
  connect( mp_actReadMameInfo, SIGNAL( activated() ), this, SLOT( slotReadMameInfoActivated() ) );
  mp_actReadMameInfo->addTo( mp_mameMenu );

  mp_mameMenu->insertSeparator();

  mp_actGameOptions = new QAction( trUtf8( "Default options" ),
                                 trUtf8( "Default options..." ),
                                 0, this, "actGameOptions" );
  mp_actGameOptions->setAccel( Qt::Key_F7 );
  connect( mp_actGameOptions, SIGNAL( activated() ), this, SLOT( slotGameOptionsActivated() ) );
  mp_actGameOptions->addTo( mp_mameMenu );

  mp_actMamePaths = new QAction( trUtf8( "Paths" ), trUtf8( "Paths..." ),
                                 0, this, "actMamePaths" );
  mp_actMamePaths->setAccel( Qt::Key_F8 );
  connect( mp_actMamePaths, SIGNAL( activated() ), this, SLOT( slotMamePathsActivated() ) );
  mp_actMamePaths->addTo( mp_mameMenu );

  menuBar()->insertItem( trUtf8( "General" ), pGeneralMenu );
  menuBar()->insertItem( trUtf8( "Filters" ), mp_filterMenu );
  menuBar()->insertItem( trUtf8( "Mame" ), mp_mameMenu );
  menuBar()->insertItem( trUtf8( "Help" ), pHelpMenu );

  connect( lvGames, SIGNAL( selectionChanged( QListViewItem* ) ),
    this, SLOT( slotGameSelected( QListViewItem* ) ) );
  connect( lvGames, SIGNAL( doubleClicked( QListViewItem* ) ),
    this, SLOT( slotStartGameSelected( QListViewItem* ) ) );
  connect( lvGames, SIGNAL( returnPressed( QListViewItem* ) ),
    this, SLOT( slotStartGameSelected( QListViewItem* ) ) );
  connect( lvGames, SIGNAL( rightButtonPressed( QListViewItem*, const QPoint&, int ) ),
    this, SLOT( slotRightButtonPressed( QListViewItem*, const QPoint&, int ) ) );
  connect( comboCategories, SIGNAL( activated( const QString& ) ),
    this, SLOT( slotCategorySelected( const QString& ) ) );
  connect( bChangePreview, SIGNAL( clicked() ),
    this, SLOT( slotChangePreviewMode() ) );

  mp_options->setGroup( G_MAME );
  changeFilter( mp_options->getNumber( O_MAME_GAME_FILTER, 0 ) );
  m_previewMode = (PreviewMode)mp_options->getNumber( O_MAME_PREVIEW_MODE, 0 );
  updatePreview();

  QTimer::singleShot( 500, this, SLOT( slotLoadGamesList() ) );
}

WndMainWindow::~WndMainWindow()
{
  mp_games->saveFavorites( mp_options->mameFavoritesFilePath() );

  mp_options->setGroup( G_MAME );
  mp_options->setNumber( O_MAME_GAME_FILTER, mp_games->filter() );
  mp_options->setNumber( O_MAME_PREVIEW_MODE, (int)m_previewMode );

  delete mp_options;
  delete mp_games;
}

void WndMainWindow::initGamesListView()
{
  lvGames->setAllColumnsShowFocus( true );
  lvGames->setShowSortIndicator( true );

  for( int i = 0; i < GameItem::countAttributes(); i++ )
  {
    lvGames->addColumn( GameItem::attributeName( i ), GameItem::attributeWidth( i ) );
  }
}

void WndMainWindow::activateProgressBar( int total_steps )
{
  if( mp_progress )
  {
    mp_progress->close();
    delete mp_progress;
    mp_progress = 0;
  }

  m_progressCancelled = false;

  if( total_steps <= 0 )
  {
    qDebug( "ProgressDialog cannot set number of steps is <= 0." );
    return;
  }

  mp_progress = new QProgressDialog( trUtf8( "Please wait..." ), trUtf8( "Cancel" ),
                                     total_steps, this, "ProgressDialog" );
  mp_progress->setCaption( "KAF" );
  mp_progress->setIcon( Pixmaps::getMameIcon() );
  mp_progress->setMinimumDuration( 2000 );
  mp_progress->setAutoClose( true );
  connect( mp_progress, SIGNAL( cancelled() ), this, SLOT( slotProgressCancelled() ) );
}

void WndMainWindow::advanceProgressBar( int steps )
{
  if( !mp_progress )
  {
    //qDebug( "There is no ProgressDialog now... something is gone wrong." );
    return;
  }

  if( steps <= 0 )
  {
    qDebug( "ProgressDialog cannot advance if number of steps is <= 0." );
    return;
  }

  mp_progress->setProgress( steps );

  if( steps >= mp_progress->totalSteps() )
  {
    mp_progress->reset();
    delete mp_progress;
    mp_progress = 0;
  }
}

void WndMainWindow::slotProgressCancelled()
{
  m_progressCancelled = true;
}

void WndMainWindow::setMessage( const QString& txt )
{
  statusBar()->message( txt );
}

void WndMainWindow::setStandardMessage()
{
  mp_options->setGroup( G_MAME );

  QString sTmp = "";

  QString sVersion = mp_options->getString( O_MAME_VERSION, "" );

  if( sVersion.isEmpty() )
    sTmp.sprintf( trUtf8("%d of %d games are available" ),
           mp_games->availables(), mp_games->total() );
  else
    sTmp.sprintf( trUtf8("MaME %s (%s): %d of %d games are available in version %s" ),
      sVersion.latin1(), mp_options->getString( O_MAME_RENDER, "..." ).latin1(),
      mp_games->availables(), mp_games->total(),
      mp_options->getString( O_MAME_GAMES_LIST_VERSION, "..." ).latin1() );

  setMessage( sTmp );
}

void WndMainWindow::setStandardMessage( Game* pGame )
{
  QString sTmp = "";

  sTmp.sprintf( trUtf8( "%s  [%s - %s - %s.zip]" ),
    pGame->description().latin1(), pGame->manufacturer().latin1(),
    pGame->year().latin1(), pGame->name().latin1() );

  setMessage( sTmp );
}

void WndMainWindow::updateLabelVersion()
{
  mp_options->setGroup( G_MAME );

  QString sVersion = mp_options->getString( O_MAME_VERSION, "" );

  if( sVersion.isEmpty() )
  {
    setCaption( KafDefaultTitle );
    return;
  }

  QString sTmp = "";

  sTmp.sprintf( trUtf8("%s - MaME version %s (%s)" ), KafDefaultTitle.latin1(),
      sVersion.latin1(), mp_options->getString( O_MAME_RENDER, "..." ).latin1() );

  setCaption( sTmp );
}

void WndMainWindow::slotMamePathsActivated()
{
  WndMamePaths* pDialog = new WndMamePaths( mp_options, this );
  pDialog->show();
  pDialog->setFixedSize( pDialog->size() );
  pDialog->exec();

  setStandardMessage();

  if( pDialog->mamePathIsChanged() )
    QTimer::singleShot( 100, this, SLOT( slotCheckMameVersion() ) );

  delete pDialog;
}

void WndMainWindow::slotRefreshGamesList()
{
  qDebug( "Refreshing games list..." );

  mp_games->refreshFiltered();

  setStandardMessage();

  setGamesListTitle();

  qDebug( "Games list refreshed." );
}

void WndMainWindow::slotBuildGamesListActivated()
{
  if( !mameBinaryExists() )
    return;

  setMessage( trUtf8("Building games list..." ) );

  mp_games->clear();

  slotChangeFilter( 0 );

  WndBuildGamesList *pBuildDialog = new WndBuildGamesList( mp_options, this );
  pBuildDialog->show();
  pBuildDialog->setFixedSize( pBuildDialog->size() );
  pBuildDialog->exec();
  delete pBuildDialog;

  QTimer::singleShot( 500, this, SLOT( slotLoadGamesList() ) );
}

void WndMainWindow::slotSearchForAvailables()
{
  activateProgressBar( mp_games->total() );
  mp_options->setGroup( G_MAME );
  QString sGameDir = mp_options->getString( O_MAME_ROMS_DIR, "/usr/local/share/xmame/roms" );
  sGameDir += "/";

  int iCount = 0;
  QString sGameName = "";

  QListViewItemIterator it( lvGames );

  while( it.current() )
  {
    iCount++;
    GameItem* item = (GameItem*)(it.current());
    sGameName = sGameDir;
    sGameName += item->game()->name();
    sGameName += ".zip";

    QFile file( sGameName );

    if( file.exists() )
      mp_games->setAvailable( item );
    else
      mp_games->unsetAvailable( item );

    if( mp_options->checkProgressTick( iCount ) )
      advanceProgressBar( iCount );

    if( mp_options->checkEventTick( iCount ) )
      qApp->processEvents();

    if( m_progressCancelled )
      break;

    ++it;
  }

  saveGamesList();

  advanceProgressBar( mp_games->total() );

  slotRefreshGamesList();

  updateLabelVersion();
}

void WndMainWindow::slotSearchForAvailablesActivated()
{
  if( gamesListIsEmpty() )
    return;

  if( !romsDirectoryExists() )
  {
    setMessage( trUtf8( "Searching for available games: interrupted" ) );
    return;
  }

  setMessage( trUtf8( "Searching for available games..." ) );

  QTimer::singleShot( 100, this, SLOT( slotSearchForAvailables() ) );
}

void WndMainWindow::slotShowAbout()
{
  QString sAbout = "";
  sAbout.sprintf( trUtf8( "KAF - Khelben's Arcade Frontend\n\n"
    "version: %s, based on xmame %s\n"
    "\nwritten by Marco Mastroddi\n\n"
    "e-mail: marco.mastroddi@libero.it"),
    Options::getKafVersion().latin1(),
    Options::getMameBasedVersion().latin1() );
  QMessageBox::about( this, trUtf8("KAF - About"), sAbout );
  setStandardMessage();
}

void WndMainWindow::showPreview( Game* pGame )
{
  if( !pGame )
  {
    labelPreview->setPixmap( Pixmaps::getMameBigIcon() );
    return;
  }

  mp_options->setGroup( G_MAME );

  QString sPreviewPath = "";
  QString sPreviewDir = "";

  switch( m_previewMode )
  {
  case FlyersPreview:
    sPreviewDir = mp_options->getString( O_MAME_FLYERS_DIR, "/usr/local/share/xmame/flyers" );
    break;
  default:
    sPreviewDir = mp_options->getString( O_MAME_SNAPS_DIR, "/usr/local/share/xmame/snap" );
  }
  sPreviewDir += "/";

  sPreviewPath = sPreviewDir + pGame->name() + mp_options->mamePreviewFileType();

  QFile file( sPreviewPath );
  if( file.exists() )
  {
    labelPreview->setPixmap( QPixmap( sPreviewPath ) );
    return;
  }

  /* Clone preview */
  if( pGame->hasFlag( Game::Clone ) )
  {
    sPreviewPath = sPreviewDir + pGame->cloneof() + mp_options->mamePreviewFileType();
    file.setName( sPreviewPath );
    if( file.exists() )
    {
      labelPreview->setPixmap( QPixmap( sPreviewPath ) );
      return;
    }
  }

  labelPreview->setPixmap( Pixmaps::getMameBigIcon() );
  qDebug( "Unable to load preview of the game '%s'.", pGame->name().latin1() );
}

void WndMainWindow::slotGameSelected( QListViewItem* item )
{
  if( !item )
  {
    qDebug( "Invalid item passed in function slotGameSelected()." );
    return;
  }

  GameItem* pGameItem = (GameItem*)item;

  Game* pGame = pGameItem->game();
  if( !pGame )
  {
    QMessageBox::warning( this, trUtf8( "KAF - Warning" ),
      trUtf8( "Invalid game found in list." ), "Ok" );
    delete item;
    return;
  }

  showPreview( pGame );

  mp_options->setGroup( G_MAME );
  mp_options->setString( O_MAME_LAST_GAME, pGame->name() );

  if( mp_info )
  {
    mp_info->slotShowInfo( pGame );
  }

  setStandardMessage( pGame );
}

void WndMainWindow::slotLoadGamesList()
{
  setMessage( trUtf8( "Loading games list... please wait" ) );

  mp_games->clear();

  setGamesListTitle();

  if( !loadGamesList() )
  {
    setMessage( trUtf8( "Ready" ) );
    return;
  }

  qApp->processEvents();

  slotCheckMameVersion();

  mp_options->setGroup( G_MAME );
  QString sLastSelectedGame = mp_options->getString( O_MAME_LAST_GAME, "" );
  if( !sLastSelectedGame.isEmpty() )
  {
    QListViewItemIterator it( lvGames );
    while( it.current() )
    {
      GameItem* item = (GameItem*)(it.current());
      if( item->game()->name() == sLastSelectedGame )
      {
        if( (it.current())->isVisible() )
        {
          lvGames->setSelected( it.current(), true );
          lvGames->ensureItemVisible( it.current() );
        }
        break;
      }
      ++it;
    }
  }

  qApp->processEvents();

  mp_options->setGroup( G_GENERAL );
  if( mp_options->getBool( O_LOAD_GAMES_INFO, false ) )
  {
    slotReadMameInfo();
  }

  setStandardMessage();
}

void WndMainWindow::slotCheckMameVersion()
{
  if( !mameBinaryExists() )
    return;

  setMessage( trUtf8( "Checking MaME version..." ) );

  MameProcess *mp = new MameProcess( mp_options );
  int iRet = mp->checkVersion();
  delete mp;

  updateLabelVersion();
  setStandardMessage();

  mp_options->setGroup( G_MAME );

  if( iRet == -1 )
  {
    qWarning( "File %s not found. Unable to check MaME version.",
      mp_options->getString( O_MAME_EXE, "/usr/local/bin/xmame" ).latin1() );
    return;
  }

  QString sGamesListVersion = mp_options->getString( O_MAME_GAMES_LIST_VERSION, "" );

  if( sGamesListVersion != mp_options->getString( O_MAME_VERSION, "" ) )
  {
    /* Games list version changed */
    switch( QMessageBox::warning( this, trUtf8( "KAF - Warning" ),
                                trUtf8( "The version of the games list was not the same of the\n"
                                        "current MaME binary. Do you want to rebuild games list?" ),
                                trUtf8( "Build..." ),
                                trUtf8( "Ignore" ) ) )
    {
    case 0:
      /* Build games list */
      slotBuildGamesListActivated();
      return;
      break;
    default:
      /* Ignore */
      break;
    }
  }
}

bool WndMainWindow::loadGamesList()
{
  QString sFile = mp_options->mameTxtFilePath();
  QFile file( sFile );
  if( !file.open( IO_ReadOnly ) )
    return false;

  QTextStream tsGames( &file );

  if( !loadGamesListHeader( &tsGames ) )
  {
    qDebug( "Missing GamesList header." );
  }

  mp_options->setGroup( G_MAME );
  activateProgressBar( mp_options->getNumber( O_MAME_GAMES_SUPPORTED, 0 ) );

  Game* pGame = 0;
  int iCount = 0;

  while( !tsGames.atEnd() )
  {
    iCount++;
    pGame = new Game();
    if( !pGame->load( &tsGames ) )
      delete pGame;
    else
      mp_games->addGame( pGame );

    if( mp_options->checkProgressTick( iCount ) )
      advanceProgressBar( iCount );

    if( mp_options->checkEventTick( iCount ) )
      qApp->processEvents();

    if( m_progressCancelled )
    {
      qDebug( "Loading games list... cancelled." );
      break;
    }
  }

  updateCategories( true );

  advanceProgressBar( mp_options->getNumber( O_MAME_GAMES_SUPPORTED, 0 ) );

  file.close();

  return true;
}

bool WndMainWindow::saveGamesList()
{
  QFile fileGames( mp_options->mameTxtFilePath() );
  if( !fileGames.open( IO_WriteOnly ) )
    return false;

  QTextStream tsGames( &fileGames );

  mp_options->setGroup( G_MAME );
  mp_options->setNumber( O_MAME_GAMES_SUPPORTED, mp_games->total() );

  saveGamesListHeader( &tsGames );

  QListViewItemIterator it( lvGames );
  while( it.current() )
  {
    GameItem* item = (GameItem*)(it.current());
    item->game()->save( &tsGames );
    ++it;
  }

  fileGames.close();

  return true;
}

bool WndMainWindow::loadGamesListHeader( QTextStream* pts )
{
  QString sHeader = pts->readLine();

  return mp_options->setMameGamesListHeader( sHeader );
}

void WndMainWindow::saveGamesListHeader( QTextStream* pts )
{
  QString sHeader = mp_options->mameGamesListHeader();
  (*pts) << sHeader << endl;
}

void WndMainWindow::slotGameOptionsActivated()
{
  WndGameOptions *pDialog = new WndGameOptions( mp_options, this );
  pDialog->show();
  //pDialog->setFixedSize( pDialog->size() );
  pDialog->exec();
  delete pDialog;
  setStandardMessage();
}

void WndMainWindow::slotStartGame()
{
  if( !lvGames->selectedItem() )
  {
    QMessageBox::information( this, trUtf8( "KAF - Information" ),
      trUtf8( "Please select the game do you want to play from the list." ), "Ok" );
    return;
  }

  slotStartGameSelected( lvGames->selectedItem() );
}

void WndMainWindow::slotStartGameSelected( QListViewItem* item )
{
  if( !item )
  {
    qWarning( "Item not found. Is it a bug?" );
    return;
  }

  if( !mameBinaryExists() )
    return;

  GameItem *pGameItem = (GameItem*)item;
  Game* pGame = pGameItem->game();

  WndMameProcess *pDialog = new WndMameProcess( pGame, mp_options );
  hide();
  pDialog->exec();
  show();
  delete pDialog;

  setStandardMessage();
}

void WndMainWindow::setGamesListTitle()
{
  QString sTitle = "";

  switch( mp_games->filter() )
  {
  case Game::Available:
    sTitle = trUtf8( "Available Games" );
    break;
  case Game::Clone:
    sTitle = trUtf8( "Clone Games" );
    break;
  case Game::Favorite:
    sTitle = trUtf8( "Favorite Games" );
    break;
  case Game::NeoGeo:
    sTitle = trUtf8( "NeoGeo Games" );
    break;
  case Game::Original:
    sTitle = trUtf8( "Original Games" );
    break;
  case Game::Unavailable:
    sTitle = trUtf8( "Unavailable Games" );
    break;
  case Game::Raster:
    sTitle = trUtf8( "Raster Games" );
    break;
  case Game::Vector:
    sTitle = trUtf8( "Vector Games" );
    break;
  default:
    sTitle = trUtf8( "All Games" );
  }

  gbGamesList->setTitle( sTitle );
}

void WndMainWindow::changeFilter( int id )
{
  if( mp_games->filter() == (unsigned int)id )
    return;

  mp_filterMenu->setItemChecked( mp_games->filter(), false );
  mp_filterMenu->setItemChecked( id, true );

  mp_games->setFilter( id );

  setGamesListTitle();
}

void WndMainWindow::slotChangeFilter( int id )
{
  changeFilter( id );
  slotRefreshGamesList();
  showItemSelected();
}

void WndMainWindow::slotRightButtonPressed( QListViewItem* item, const QPoint& pos, int )
{
  if( !item )
  {
    qDebug( "Item not found. Maybe the list is empty." );
    return;
  }

  GameItem *pGameItem = (GameItem*)item;
  Game* pGame = pGameItem->game();

  QPopupMenu *pPopup = new QPopupMenu( this, "rightButtonPopupMenu" );

  if( !pGame->hasFlag( Game::Available ) )
  {
    pPopup->insertItem( trUtf8( "This game is not available" ) );
  }
  else
  {
    QString sTmp = "";
    sTmp.sprintf( trUtf8( "Play %s..." ), pGame->description().latin1() );
    pPopup->insertItem( sTmp, this, SLOT( slotStartGame() ) );
    pPopup->insertSeparator();
    if( pGame->hasFlag( Game::Favorite ) )
      pPopup->insertItem( trUtf8( "Remove game from favorites" ), this, SLOT( slotChangeFavoriteOptionForGame() ) );
    else
      pPopup->insertItem( trUtf8( "Add game to favorites" ), this, SLOT( slotChangeFavoriteOptionForGame() ) );
  }

  pPopup->insertSeparator();

  pPopup->insertItem( trUtf8( "Show game information..." ), this, SLOT( slotShowInfoWidget() ) );

  pPopup->insertItem( trUtf8( "Options..." ), this, SLOT( slotGameOptionsActivated() ) );

  pPopup->exec( pos );
  delete pPopup;
}

void WndMainWindow::slotChangeFavoriteOptionForGame()
{
  QListViewItem* item = lvGames->selectedItem();

  if( !item )
  {
    qWarning( "Item not found. Is it a bug?" );
    return;
  }

  GameItem *pGameItem = (GameItem*)item;

  if( pGameItem->game()->hasFlag( Game::Favorite ) )
    mp_games->unsetFavorite( pGameItem );
  else
    mp_games->setFavorite( pGameItem );
}

void WndMainWindow::slotReadMameInfoActivated()
{
  if( gamesListIsEmpty() )
    return;

  setMessage( trUtf8( "Reading games information..." ) );

  QTimer::singleShot( 100, this, SLOT( slotReadMameInfo() ) );
}

void WndMainWindow::slotReadMameInfo()
{
  mp_options->setGroup( G_MAME );
  QString sInfoPath = mp_options->getString( O_MAME_INFO_FILE, "/usr/local/share/xmame/mameinfo.dat" );
  QString sTmp = "";

  QFile file( sInfoPath );

  if( !file.exists() )
  {
    sTmp.sprintf( trUtf8( "File '%s' not found." ), sInfoPath.latin1() );
    QMessageBox::warning( this, trUtf8( "KAF - Warning" ), sTmp, "Ok" );
    setMessage( sTmp );
    return;
  }

  if( !file.open( IO_ReadOnly ) )
  {
    sTmp.sprintf( trUtf8( "Unable to open the file '%s'." ), sInfoPath.latin1() );
    QMessageBox::warning( this, trUtf8( "KAF - Warning" ), sTmp, "Ok" );
    setMessage( sTmp );
    return;
  }

  QString sGameName = "";
  QString sBuffer = "";
  Game *pGame = NULL;
  QTextStream ts( &file );
  int iCount = 0;
  activateProgressBar( mp_games->total() );

  while( !ts.atEnd() )
  {
    sTmp = ts.readLine();

    if( sTmp.isNull() || sTmp.isEmpty() )
      continue;

    if( sTmp.startsWith( "#" ) ) // Skip comment
      continue;

    if( sTmp.startsWith( "$info=" ) )
    {
      /* Get name */
      sGameName = sTmp.mid( 6, sTmp.length() );
      if( sGameName.isNull() || sGameName.isEmpty() )
        sGameName = "";
      continue;
    }

    if( sTmp.startsWith( "$mame" ) && !sGameName.isEmpty() )
    {
      sBuffer = "";
      pGame = mp_games->game( sGameName );
      if( pGame == NULL )
      {
        qDebug( "Found info for the game '%s' but it is not in the list.",
          sGameName.latin1() );
        sGameName = "";
      }
      continue;
    }

    if( sTmp.startsWith( "Category:" ) && pGame != NULL )
    {
      sTmp = sTmp.mid( 9, sTmp.length() );
      if( sTmp.isNull() || sTmp.isEmpty() )
        continue;

      pGame->category() = sTmp.simplifyWhiteSpace();
      mp_games->addCategory( pGame->category() );
      continue;
    }

    if( sTmp.startsWith( "$end" ) && pGame != NULL )
    {
      iCount++;
      pGame->info() = sBuffer;
      //qDebug( "Save info for game '%s'.", sGameName.latin1() );
      pGame = NULL;
      sGameName = "";

      if( mp_options->checkProgressTick( iCount ) )
        advanceProgressBar( iCount );

      if( mp_options->checkEventTick( iCount ) )
        qApp->processEvents();

      continue;
    }

    if( pGame != NULL )
    {
      sBuffer += sTmp;
      sBuffer += "\n";
    }
  }

  file.close();
  advanceProgressBar( mp_games->total() );
  qApp->processEvents();

  updateCategories( true );

  activateProgressBar( mp_games->total() );
  QListViewItemIterator it( lvGames );
  GameItem* gi = NULL;
  pGame = NULL;
  iCount = 0;
  while( it.current() )
  {
    iCount++;
    gi = (GameItem*)it.current();
    if( gi->game()->hasFlag( Game::Clone ) )
    {
      pGame = mp_games->game( gi->game()->cloneof() );
      if( pGame != NULL )
      {
        gi->game()->category() = pGame->category();
      }
    }

    if( iCount < (mp_games->total() - 1) )
    {
      if( mp_options->checkProgressTick( iCount ) )
        advanceProgressBar( iCount );

      if( mp_options->checkEventTick( iCount ) )
        qApp->processEvents();
    }

    ++it;
  }

  advanceProgressBar( mp_games->total() - 1 );
  qApp->processEvents();

  mp_games->refreshAllGames();

  advanceProgressBar( mp_games->total() );
  qApp->processEvents();

  saveGamesList();

  setMessage( trUtf8( "Games information read." ) );
}

void WndMainWindow::slotShowInfoWidget()
{
  if( !mp_info )
  {
    qDebug( "Creating new information widget." );
    mp_info = new WndInfo( mp_options, this, "Info Window" );
    connect( mp_info, SIGNAL( closing() ), this, SLOT( slotDeleteInfoWidget() ) );
    mp_info->show();
  }

  if( !lvGames->selectedItem() )
  {
    qWarning( "Game selected has lost his focus, cannot display information." );
    return;
  }

  GameItem *pGameItem = (GameItem*)(lvGames->selectedItem());

  mp_info->slotShowInfo( pGameItem->game() );

}

void WndMainWindow::slotDeleteInfoWidget()
{
  if( mp_info )
  {
    qDebug( "Delete information widget." );
    disconnect( mp_info, SIGNAL( closing() ), this, SLOT( slotDeleteInfoWidget() ) );
    delete mp_info;
    mp_info = 0;
  }
}

void WndMainWindow::slotShowMameInfo()
{
  mp_options->setGroup( G_MAME );

  QString sVersion = mp_options->getString( O_MAME_VERSION, "" );
  QString sTmp = "";

  if( sVersion.isEmpty() )
  {
    sTmp = trUtf8( "MaME informations are unavailable." );
  }
  else
  {
    sTmp.sprintf( trUtf8("MaME version %s (%s) built in %s\n\n%d of %d games are available" ),
      sVersion.latin1(), mp_options->getString( O_MAME_RENDER, "..." ).latin1(),
      mp_options->getString( O_MAME_BUILD, "unknown date" ).latin1(),
      mp_games->availables(), mp_games->total() );
  }

  QMessageBox::information( this, trUtf8( "KAF - Information" ), sTmp );
  setStandardMessage();
}

void WndMainWindow::updateCategories( bool sort )
{
  if( sort )
    mp_games->sortCategories();
  comboCategories->clear();
  comboCategories->insertStringList( mp_games->categories() );
}

void WndMainWindow::slotCategorySelected( const QString& cat_name )
{
  if( mp_games->categoryFilter() != cat_name )
  {
    mp_games->setCategoryFilter( cat_name );
    mp_games->refreshFiltered();
    showItemSelected();
  }
}

bool WndMainWindow::gamesListIsEmpty()
{
  if( mp_games->total() == 0 )
  {
    QMessageBox::information( this, trUtf8( "KAF - Information" ),
      trUtf8( "Your games list is empty. Please rebuild a new one" ), "Ok" );
    return true;
  }

  return false;
}

bool WndMainWindow::romsDirectoryExists()
{
  mp_options->setGroup( G_MAME );

  QString sGameDir = mp_options->getString( O_MAME_ROMS_DIR, "/usr/local/share/xmame/roms" );

  QDir dir( sGameDir );

  if( !dir.exists() )
  {
    QString sTmp = "";
    sTmp.sprintf( trUtf8( "Unable to open directory '%s'." ), sGameDir.latin1() );
    QMessageBox::warning( this, trUtf8( "KAF - Warning" ), sTmp, "Ok" );
    return false;
  }

  return true;
}

bool WndMainWindow::mameBinaryExists()
{
  mp_options->setGroup( G_MAME );

  QString sExePath = mp_options->getString( O_MAME_EXE, "/usr/local/bin/xmame" );

  QFile file( sExePath );

  if( !file.exists() )
  {
    QString sTmp = trUtf8( "Mame binary '" );
    sTmp += sExePath;
    sTmp += trUtf8( "' not found.\nPlease check the path in your configuration." );
    QMessageBox::warning( this, trUtf8( "KAF - Warning" ), sTmp, "Ok" );
    return false;
  }

  return true;
}

void WndMainWindow::slotChangePreviewMode()
{
  if( m_previewMode == FlyersPreview )
  {
    m_previewMode = SnapshotsPreview;
  }
  else
  {
    m_previewMode = FlyersPreview;
  }

  updatePreview();

  slotGameSelected( lvGames->selectedItem() );
}

void WndMainWindow::updatePreview()
{
  QString sPreviewTitle = "";

  switch( m_previewMode )
  {
  case FlyersPreview:
    sPreviewTitle = trUtf8( "Flyers" );
    break;
  default:
    sPreviewTitle = trUtf8( "Snapshots" );
  }

  gbPreview->setTitle( sPreviewTitle );
}

void WndMainWindow::showItemSelected()
{
  QListViewItem* item = lvGames->selectedItem();
  if( !item || ( item && !item->isVisible() ) )
  {
    lvGames->setSelected( item, false );
    showPreview( 0 );
  }
  else
    lvGames->ensureItemVisible( item );
}

void WndMainWindow::slotShowPreferences()
{
  WndPreferences* pDialog = new WndPreferences( mp_options, this );
  pDialog->show();
  pDialog->setFixedSize( pDialog->size() );
  pDialog->exec();
  delete pDialog;
}
